import pytest
from circles_text_block_local.text_blocks import TextBlocks

def test_valid_regex():
    blocks = [(3, 
    '''Sydney Lauer
    sydlauer@umich.edu
    9252853371
    0553083627
    Ann Arbor, Michigan
    '''),  (4,
    '''Sydney Lauer
    0301/2003
    9252853371
    srlauer3103@gmail.com
    March 1 2003
    ''')]
    classifier = TextBlocks(blocks)
    
    fields = classifier.extract_fields()
    expected = {'Email': [' sydlauer@umich.edu', ' srlauer3103@gmail.com'], 'Phone Number': [' 9252853371']}
    assert fields == expected

def test_invalid_regex():
    blocks = [
        (4, '''InvalidRegexTest
                invalidemail@adress
                9252853371
                '''),
    ]
    classifier = TextBlocks(blocks)

    fields = classifier.extract_fields()
    expected = {'Phone Number': [' 9252853371']}
    assert fields == expected

def test_no_matches():
    blocks = [
        (6, '''NoMatchTest
                no@email
                No phone number
                '''),
    ]
    classifier = TextBlocks(blocks)
    fields = classifier.extract_fields()
    assert fields == {}

def test_empty_block():
    blocks = [
        (7, ''),
    ]
    classifier = TextBlocks(blocks)
    fields = classifier.extract_fields()
    assert fields == {}

def test_newline_handling():
    blocks = [
        (8, '''TestNewlineHandling
                test@example.com
                12345\n67890
                Line1\nLine2
                '''),
    ]
    classifier = TextBlocks(blocks)
    fields = classifier.extract_fields()
    expected = {'Email': [' test@example.com']}
    assert fields == expected

def test_other_block_types():
    blocks = [
            (1, '''BlockType1Test
                    test1@example.com
                    111-111-1111
                    www.websiteexample.com
                 '''),
            (2, '''BlockType2Test
                    test2@example.com
                    222-222-2222
                    website.com
                 '''),
        ]
    classifier = TextBlocks(blocks)
    fields = classifier.extract_fields()
    expected = {'Email': [' test1@example.com', ' test2@example.com']}
    assert fields == expected







import setuptools
# Each Python project should have pyproject.toml or setup.py
# TODO: Please create pyproject.toml instead of setup.py (delete the setup.py)
# used by python -m build
# ```python -m build``` needs pyproject.toml or setup.py
# The need for setup.py is changing as of poetry 1.1.0 (including current pre-release) as we have moved away from needing to generate a setup.py file to enable editable installs - We might able to delete this file in the near future
setuptools.setup(
     # TODO: Please update the name and delete this line i.e. XXX-local or XXX-remote (without the -python-package suffix). Only lower case
     name='text-block-local',  
     version='0.0.2',
     author="Circles",
     author_email="info@circles.life",
     # TODO: Please update the description and delete this line
     description="PyPI Package for Circles Text-Block-Local Python",
     # TODO: Please update the long description and delete this line    
     long_description="This is a package for sharing common XXX function used in different repositories",
     long_description_content_type="text/markdown",
     url="https://github.com/circles",
     packages=setuptools.find_packages(),
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: Other/Proprietary License",
         "Operating System :: OS Independent",
     ],
 )



from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python import database 
from logger_local_python_package.localLogger import _local_logger as logger_local

import re

class TextBlocks:

  def __init__(self, text_blocks=None):
    if text_blocks is None:
      #use text blocks in text_block_table
      self.text_blocks = self.get_text_blocks()
    else:
      #use user_input for text_blocks
      self.text_blocks = text_blocks
      
  # Connect to the database
  def db_connection(self):
    database_conn = database.database()
    db = database_conn.connect_to_database()
    return db
    
  def get_block_fields(self, block_type_id):
    logger_local.start("Getting regex and field_id from block_id ...")
    
    conn = self.db_connection()
    if conn: 
      cursor = conn.cursor()

      cursor.execute("SELECT regex, field_id FROM field.block_type_field_table WHERE block_type_id = %s OR block_type_id IS NULL" % block_type_id)
      block_fields = dict ((regex, field_id) for regex, field_id in cursor.fetchall())

      cursor.close()
      conn.close()
      logger_local.end("Regex and field ids retrieved", object = { 'block_fields': block_fields})    
      return block_fields

    logger_local.info("Error connecting to the database")

  def get_fields(self):
    logger_local.start("Getting field ids and names ...")    

    conn = self.db_connection()
    if conn: 
      cursor = conn.cursor()
      
      cursor.execute("SELECT id, name FROM field.field_table")
      fields = dict ((id, name) for id, name in cursor.fetchall())

      cursor.close()
      conn.close()
      logger_local.end("Field names and ids retrieved", object = { 'fields': fields})    
      return fields

    logger_local.info("Error connecting to the database")

  def get_block_types(self):
    logger_local.start("Getting block type ids and names ...")    

    conn = self.db_connection()
    if conn: 
      cursor = conn.cursor()

      cursor.execute("SELECT name, block_type_id FROM block_type.block_type_ml_table")
      block_types = dict ((id, name) for name, id in cursor.fetchall())

      cursor.close()
      conn.close()
      logger_local.end("Block types retrieved", object = { 'block_types': block_types})    
      return block_types

    logger_local.info("Error connecting to the database")

  def get_text_blocks(self):
    logger_local.start("Getting text blocks from text_block_table ...")    

    conn = self.db_connection()
    if conn: 
      cursor = conn.cursor()

      cursor.execute("SELECT text_block_type_id, text FROM text_block.text_block_table")
      text_blocks = cursor.fetchall()

      cursor.close()
      conn.close()
      logger_local.end("Text blocks retrieved", object = { 'text_blocks': text_blocks})    
      return text_blocks

    logger_local.info("Error connecting to the database")

  def extract_fields(self):
    logger_local.start("Beginning text-block extracting process ...")    
    
    fields_dict = {}

    #get dictionary of field ids and names
    fields = self.get_fields()
    if fields is None:
        return "Error retreieving fields dictionary"

    #loop through set of text blocks
    for block in self.text_blocks:
      (block_id , text) = (block[0], block[1])

      #reformat text 
      text = text.replace("\n", " ")

      #get block id and corresponding regex/fields 
      block_fields = self.get_block_fields(block_id)
      if block_fields is None:
            return "Error retreieving block fields dictionary"

      #extract fields for each regex expression 
      for regex in block_fields:
        if regex:    
          #check the regex is valid    
          try: 
            re.compile(regex)
            matches = re.findall(regex, text)
            if matches and block_fields[regex] != 0:
              fields_dict.setdefault(fields[block_fields[regex]], []).extend(matches)
      
          #print error if regex is invalid
          except re.error as e: 
            logger_local.exception(str("Invalid regex: ", regex), object=e)

    logger_local.end("Finished extracting all fields", object = {'field_dictionary': fields_dict}) 
    return fields_dict   
